@@ -2,9 +2,6 @@ from PyQt5 import QtCore
 from napari._qt.qthreading import thread_worker
 from PyQt5.QtWidgets import QCheckBox, QComboBox, QDoubleSpinBox, QFileDialog, QLabel, QSpinBox, QVBoxLayout
 from napari_plugin_engine import napari_hook_implementation
-from numpy.lib.function_base import append
-from numpy.lib.shape_base import expand_dims
-from numpy.lib.type_check import imag
 from qtpy.QtWidgets import QWidget, QGridLayout, QPushButton
 import numpy as np
 import tifffile
@@ -12,6 +9,7 @@ from napari_live_recording.Cameras import *
 from napari_live_recording.Functions import *
 from collections import deque
 from dask_image.imread import imread
+from copy import deepcopy
 
 
 class LiveRecording(QWidget):
@@ -109,8 +107,8 @@ class LiveRecording(QWidget):
         self.camera_roi_x_offset_label.setAlignment(QtCore.Qt.AlignCenter)
 
         self.camera_roi_x_offset_spinbox = QSpinBox(self)
-        self.camera_roi_x_offset_spinbox.setRange(0, 1280)
-        self.camera_roi_x_offset_spinbox.setSingleStep(32)
+        self.camera_roi_x_offset_spinbox.setRange(0, 0)
+        self.camera_roi_x_offset_spinbox.setSingleStep(0)
         self.camera_roi_x_offset_spinbox.setValue(0)
         self.camera_roi_x_offset_spinbox.valueChanged.connect(
             self._on_offset_x_changed)
@@ -119,8 +117,8 @@ class LiveRecording(QWidget):
         self.camera_roi_y_offset_label.setAlignment(QtCore.Qt.AlignCenter)
 
         self.camera_roi_y_offset_spinbox = QSpinBox(self)
-        self.camera_roi_y_offset_spinbox.setRange(0, 864)
-        self.camera_roi_y_offset_spinbox.setSingleStep(2)
+        self.camera_roi_y_offset_spinbox.setRange(0, 0)
+        self.camera_roi_y_offset_spinbox.setSingleStep(0)
         self.camera_roi_y_offset_spinbox.setValue(0)
         self.camera_roi_y_offset_spinbox.valueChanged.connect(
             self._on_offset_y_changed)
@@ -129,9 +127,9 @@ class LiveRecording(QWidget):
         self.camera_roi_width_label.setAlignment(QtCore.Qt.AlignCenter)
 
         self.camera_roi_width_spinbox = QSpinBox(self)
-        self.camera_roi_width_spinbox.setRange(32, 1280)
-        self.camera_roi_width_spinbox.setSingleStep(32)
-        self.camera_roi_width_spinbox.setValue(1280)
+        self.camera_roi_width_spinbox.setRange(0, 0)
+        self.camera_roi_width_spinbox.setSingleStep(0)
+        self.camera_roi_width_spinbox.setValue(0)
         self.camera_roi_width_spinbox.valueChanged.connect(
             self._on_width_changed)
 
@@ -139,9 +137,9 @@ class LiveRecording(QWidget):
         self.camera_roi_height_label.setAlignment(QtCore.Qt.AlignCenter)
 
         self.camera_roi_height_spinbox = QSpinBox(self)
-        self.camera_roi_height_spinbox.setRange(4, 864)
-        self.camera_roi_height_spinbox.setSingleStep(4)
-        self.camera_roi_height_spinbox.setValue(864)
+        self.camera_roi_height_spinbox.setRange(0, 0)
+        self.camera_roi_height_spinbox.setSingleStep(0)
+        self.camera_roi_height_spinbox.setValue(0)
         self.camera_roi_height_spinbox.valueChanged.connect(
             self._on_height_changed)
 
@@ -174,10 +172,9 @@ class LiveRecording(QWidget):
         self.outer_layout.addLayout(self.roi_layout)
         self.setLayout(self.outer_layout)
 
-        self.roi = CameraROI()
+        self.roi = None
 
-        # 1 s timer for FPS in live recording
-        self.live_fps_timer.setInterval(1000)
+        self.live_fps_timer.setInterval(1000) # 1 s timer for FPS in live recording
         self.live_view_timer.setInterval(1/60)  # grants 60 FPS for viewing
 
         self.live_fps_timer.timeout.connect(self._update_frames_per_second)
@@ -227,6 +224,7 @@ class LiveRecording(QWidget):
             self.camera_pixel_type_combobox.clear()
         self.camera_connect_button.setEnabled(bool(index))
         camera_name = self.camera_selection_combobox.currentText()
+        self.camera_pixel_type_combobox.clear()
         try:
             # constructs object of class specified by camera_name
             self.camera = supported_cameras[camera_name]()
@@ -237,6 +235,7 @@ class LiveRecording(QWidget):
             else:
                 self._add_camera_exposure()
             self.camera_pixel_type_combobox.addItems(self.camera.get_available_pixel_formats())
+            self.roi = self.camera.get_sensor_range()
 
         except KeyError:  # unsupported camera found
             if self.camera_exposure_label is not None:
@@ -288,12 +287,28 @@ class LiveRecording(QWidget):
         self.camera_exposure_label = QLabel("Exposure (ms)", self)
         self.options_layout.addWidget(self.camera_exposure_label, 9, 0)
         self.camera_exposure_widget = QDoubleSpinBox(self)
-        self.camera_exposure_widget.setRange(0.01, 100)
+        self.camera_exposure_widget.setRange(0.001, 100)
         self.camera_exposure_widget.setValue(1.0)
-        self.camera_exposure_widget.setSingleStep(0.01)
+        self.camera_exposure_widget.setSingleStep(0.001)
+        self.camera_exposure_widget.setDecimals(3)
         self.camera_exposure_widget.valueChanged.connect(
             self._on_exposure_changed)
         self.options_layout.addWidget(self.camera_exposure_widget, 9, 1)
+    
+    def _initialize_roi_widgets(self, roi : CameraROI):
+        self.camera_roi_x_offset_spinbox.setRange(0, roi.width)
+        self.camera_roi_x_offset_spinbox.setSingleStep(roi.ofs_x_step)
+
+        self.camera_roi_y_offset_spinbox.setRange(0, roi.height)
+        self.camera_roi_y_offset_spinbox.setSingleStep(roi.ofs_y_step)
+
+        self.camera_roi_width_spinbox.setRange(0, roi.width)
+        self.camera_roi_width_spinbox.setSingleStep(roi.width_step)
+        self.camera_roi_width_spinbox.setValue(roi.width)
+
+        self.camera_roi_height_spinbox.setRange(0, roi.height)
+        self.camera_roi_height_spinbox.setSingleStep(roi.height_step)
+        self.camera_roi_height_spinbox.setValue(roi.height)
 
     def _on_connect_clicked(self):
         if not self.is_connect:
@@ -302,7 +317,9 @@ class LiveRecording(QWidget):
                 self.is_connect = True
                 self._set_widgets_enabled(True)
                 self.camera_pixel_type_combobox.currentTextChanged.connect(self._on_pixel_format_changed)
-                self._set_default_roi()
+                # when opening the device, the ROI settings in the GUI must be updated
+                roi = self.camera.get_roi()
+                self._initialize_roi_widgets(roi)
             else:
                 raise CameraError(f"Error in opening {self.camera.get_name()}")
         else:
@@ -310,6 +327,7 @@ class LiveRecording(QWidget):
             self.camera_connect_button.setText("Connect camera")
             self.is_connect = False
             self._set_widgets_enabled(False)
+            self._initialize_roi_widgets(CameraROI())
 
     # inspired by https://github.com/haesleinhuepf/napari-webcam
     def _update_layer(self):
@@ -317,9 +335,17 @@ class LiveRecording(QWidget):
         # to access data stored from different thread
         try:
             data = self.live_image_buffer.pop()
-            self.viewer.layers["Live recording"].data = data
+            # this happens in case there is a change from
+            # a color base to grayscale image; we need
+            # to delete the layer and refresh it with
+            # new data
+            if data.ndim != self.viewer.layers["Live recording"].data.ndim:
+                self.viewer.layers.remove("Live recording")
+                self.viewer.add_image(data, name="Live recording")
+            else:
+                self.viewer.layers["Live recording"].data = data
         except KeyError:
-            self.viewer.add_image(data, name="Live recording")
+            self.viewer.add_image(data, name="Live recording")            
         except IndexError:
             pass
 
@@ -419,14 +445,14 @@ class LiveRecording(QWidget):
         self.roi.offset_y = offset
 
     def _on_roi_change_requested(self):
-        self.camera.set_roi(self.roi)
+        self.camera.set_roi(deepcopy(self.roi))
+        # it may happen that the true ROI is not the one set by the function
+        # i.e. the Ximea performs an approximation
+        # hence we update the spinbox with the proper values
+        self._update_indexes()
 
-    def _on_roi_full_requested(self):
-        self.camera.set_full_frame()
-        self._set_default_roi()
-
-    def _set_default_roi(self):
-        self.roi = self.camera.get_sensor_range()
+    def _update_indexes(self):
+        self.roi = self.camera.get_roi()
         self.camera_roi_x_offset_spinbox.setValue(self.roi.offset_x)
         self.camera_roi_y_offset_spinbox.setValue(self.roi.offset_y)
         self.camera_roi_x_offset_spinbox.setSingleStep(self.roi.ofs_x_step)
@@ -436,7 +462,10 @@ class LiveRecording(QWidget):
         self.camera_roi_height_spinbox.setSingleStep(self.roi.height_step)
         self.camera_roi_width_spinbox.setValue(self.roi.width)
         self.camera_roi_width_spinbox.setSingleStep(self.roi.width_step)
-        self.camera.set_roi(self.roi)
+
+    def _on_roi_full_requested(self):
+        self.camera.set_full_frame()
+        self._update_indexes()
 
 
 @napari_hook_implementation
